@@ -51,8 +51,23 @@
 #include <X11/Xutil.h>
 #endif
 
+#ifdef HAVE_SYS_SOUNDCARD_H
 #include <sys/soundcard.h>
+#endif
 #ifdef HAVE_V4L 
+/* Because of some really cool feature in video4linux1, also known as
+ * 'not including sys/types.h and sys/time.h', we had to include it
+ * ourselves. In all their intelligence, these people decided to fix
+ * this in the next version (video4linux2) in such a cool way that it
+ * breaks all compilations of old stuff...
+ * The real problem is actually that linux/time.h doesn't use proper
+ * macro checks before defining types like struct timeval. The proper
+ * fix here is to either fuck the kernel header (which is what we do
+ * by defining _LINUX_TIME_H, an innocent little hack) or by fixing it
+ * upstream, which I'll consider doing later on. If you get compiler
+ * errors here, check your linux/time.h && sys/time.h header setup.
+ */
+#define _LINUX_TIME_H
 #include <linux/videodev.h>
 #else
 #define VIDEO_MODE_PAL		0
@@ -75,6 +90,118 @@
 /*#include "editlist.h" */
 #include "jpegutils.h"
 
+#ifdef	HAVE_LIBDV
+#include <libdv/dv.h>
+
+dv_decoder_t *decoder;
+int pitches[3];
+uint8_t *dv_frame[3] = {NULL,NULL,NULL};
+/*
+ * As far as I (maddog) can tell, this is what is going on with libdv-0.9
+ *  and the unpacking routine... 
+ *    o Ft/Fb refer to top/bottom scanlines (interleaved) --- each sample
+ *       is implicitly tagged by 't' or 'b' (samples are not mixed between
+ *       fields)
+ *    o Indices on Cb and Cr samples indicate the Y sample with which
+ *       they are co-sited.
+ *    o '^' indicates which samples are preserved by the unpacking
+ *
+ * libdv packs both NTSC 4:1:1 and PAL 4:2:0 into a common frame format of
+ *  packed 4:2:2 pixels as follows:
+ *
+ *
+ ***** NTSC 4:1:1 *****
+ *
+ *   libdv's 4:2:2-packed representation  (chroma repeated horizontally)
+ *
+ *Ft Y00.Cb00.Y01.Cr00.Y02.Cb00.Y03.Cr00 Y04.Cb04.Y05.Cr04.Y06.Cb04.Y07.Cr04
+ *    ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^
+ *Fb Y00.Cb00.Y01.Cr00.Y02.Cb00.Y03.Cr00 Y04.Cb04.Y05.Cr04.Y06.Cb04.Y07.Cr04
+ *    ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^
+ *Ft Y10.Cb10.Y11.Cr10.Y12.Cb10.Y13.Cr10 Y14.Cb14.Y15.Cr14.Y16.Cb14.Y17.Cr14
+ *    ^        ^        ^        ^        ^        ^        ^        ^    
+ *Fb Y10.Cb10.Y11.Cr10.Y12.Cb10.Y13.Cr10 Y14.Cb14.Y15.Cr14.Y16.Cb14.Y17.Cr14
+ *    ^        ^        ^        ^        ^        ^        ^        ^    
+ *
+ *    lavtools unpacking into 4:2:0-planar  (note lossiness)
+ *
+ *Ft  Y00.Y01.Y02.Y03.Y04.Y05.Y06.Y07...
+ *Fb  Y00.Y01.Y02.Y03.Y04.Y05.Y06.Y07...
+ *Ft  Y10.Y11.Y12.Y13.Y14.Y15.Y16.Y17...
+ *Fb  Y10.Y11.Y12.Y13.Y14.Y15.Y16.Y17...
+ *
+ *Ft  Cb00.Cb00.Cb04.Cb04...    Cb00,Cb04... are doubled
+ *Fb  Cb00.Cb00.Cb04.Cb04...    Cb10,Cb14... are ignored
+ *
+ *Ft  Cr00.Cr00.Cr04.Cr04...
+ *Fb  Cr00.Cr00.Cr04.Cr04...
+ *
+ * 
+ ***** PAL 4:2:0 *****
+ *
+ *   libdv's 4:2:2-packed representation  (chroma repeated vertically)
+ *
+ *Ft Y00.Cb00.Y01.Cr10.Y02.Cb02.Y03.Cr12 Y04.Cb04.Y05.Cr14.Y06.Cb06.Y07.Cr16
+ *    ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^
+ *Fb Y00.Cb00.Y01.Cr10.Y02.Cb02.Y03.Cr12 Y04.Cb04.Y05.Cr14.Y06.Cb06.Y07.Cr16
+ *    ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^   ^    ^
+ *Ft Y10.Cb00.Y11.Cr10.Y12.Cb02.Y13.Cr12 Y14.Cb04.Y15.Cr14.Y16.Cb06.Y17.Cr16
+ *    ^        ^        ^        ^        ^        ^        ^        ^    
+ *Fb Y10.Cb00.Y11.Cr10.Y12.Cb02.Y13.Cr12 Y14.Cb04.Y15.Cr14.Y16.Cb06.Y17.Cr16
+ *    ^        ^        ^        ^        ^        ^        ^        ^    
+ *
+ *    lavtools unpacking into 4:2:0-planar
+ *
+ *Ft  Y00.Y01.Y02.Y03.Y04.Y05.Y06.Y07...
+ *Fb  Y00.Y01.Y02.Y03.Y04.Y05.Y06.Y07...
+ *Ft  Y10.Y11.Y12.Y13.Y14.Y15.Y16.Y17...
+ *Fb  Y10.Y11.Y12.Y13.Y14.Y15.Y16.Y17...
+ *
+ *Ft  Cb00.Cb02.Cb04.Cb06...
+ *Fb  Cb00.Cb02.Cb04.Cb06...
+ *
+ *Ft  Cr10.Cr12.Cr14.Cr16...
+ *Fb  Cr10.Cr12.Cr14.Cr16...
+ *
+ */
+
+/*
+ * Unpack libdv's 4:2:2-packed into our 4:2:0-planar
+ *
+ */
+void frame_YUV422_to_YUV420P(uint8_t **output, uint8_t *input,
+			     int width, int height)
+{
+    int i, j, w2;
+    uint8_t *y, *cb, *cr;
+
+    w2 = width/2;
+    y = output[0];
+    cb = output[1];
+    cr = output[2];
+
+    for (i=0; i<height; i+=2) {
+	/* process two scanlines (one from each field, interleaved) */
+        for (j=0; j<w2; j++) {
+            /* packed YUV 422 is: Y[i] U[i] Y[i+1] V[i] */
+            *(y++) =  *(input++);
+            *(cb++) = *(input++);
+            *(y++) =  *(input++);
+            *(cr++) = *(input++);
+        }
+	/* process next two scanlines (one from each field, interleaved) */
+	for (j=0; j<w2; j++) {
+	  /* skip every second line for U and V */
+	  *(y++) = *(input++);
+	  input++;
+	  *(y++) = *(input++);
+	  input++;
+	}
+    }
+}
+
+#endif
+
 
 /* On some systems MAP_FAILED seems to be missing */
 #ifndef MAP_FAILED
@@ -121,20 +248,14 @@ typedef struct {
    long currently_processed_entry;
 #endif
 
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
    int data_format[MJPEG_MAX_BUF];
-#endif
-
    struct mjpeg_sync syncinfo[MJPEG_MAX_BUF]; /* synchronization info */
-
    unsigned long *save_list;                  /* for editing purposes */
    long save_list_len;                        /* for editing purposes */
-
    uint8_t abuff[16384];                       /* the audio buffer */
    double spas;                               /* seconds per audio sample */
    long   audio_buffer_size;                  /* audio stream buffer size */
    int    audio_mute;                         /* controls whether to currently play audio or not */
-
    int    state;                              /* playing, paused or stoppped */
    pthread_t playback_thread;                 /* the thread for the whole playback-library */
 } video_playback_setup;
@@ -342,9 +463,7 @@ static int lavplay_get_audio(lavplay_t *info, uint8_t *buff, long frame_num, int
  ******************************************************/
 
 static int lavplay_queue_next_frame(lavplay_t *info, uint8_t *vbuff,
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
 				    int data_format,
-#endif
 				    int skip_video, int skip_audio, int skip_incr)
 {
    int res, mute, i, jpeg_len1, jpeg_len2, new_buff_no;
@@ -356,9 +475,7 @@ static int lavplay_queue_next_frame(lavplay_t *info, uint8_t *vbuff,
    if (!skip_video)
    {
       if (info->flicker_reduction && editlist->video_inter &&
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
 	  data_format == DATAFORMAT_MJPG &&
-#endif	  
 	  settings->current_playback_speed <= 0)
       {
          if (settings->current_playback_speed == 0)
@@ -519,21 +636,21 @@ static int lavplay_SDL_unlock(lavplay_t *info)
 
 #ifdef HAVE_SDL
 static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
 			      int data_format,
-#endif
 			      int buf_len)
 {
    video_playback_setup *settings = (video_playback_setup *)info->settings;
+# ifdef HAVE_LIBDV
+   uint8_t *output[3];
+#endif
    EditList *editlist = info->editlist;
 
+
    if (!lavplay_SDL_lock(info)) return 0;
 
    /* decode frame to yuv */
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
    switch (data_format) {
    case DATAFORMAT_MJPG:
-#endif
    decode_jpeg_raw (jpeg_buffer, buf_len,
       editlist->video_inter>0&&info->exchange_fields?(editlist->video_inter+1)%2+1:editlist->video_inter,
       CHROMA420,
@@ -541,9 +658,7 @@ static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
       settings->yuv_overlay->pixels[0], 
       settings->yuv_overlay->pixels[2],
       settings->yuv_overlay->pixels[1]);
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
       break;
-# ifdef SUPPORT_READ_YUV420
    case DATAFORMAT_YUV420:
       memcpy(settings->yuv_overlay->pixels[0],
 	     jpeg_buffer,
@@ -555,22 +670,48 @@ static int lavplay_SDL_update(lavplay_t *info, uint8_t *jpeg_buffer,
 	     jpeg_buffer + (editlist->video_width * editlist->video_height * 5 / 4),
 	     (editlist->video_width * editlist->video_height / 4));
       break;
-# endif
-# ifdef SUPPORT_READ_DV2
+
+# ifdef HAVE_LIBDV
    case DATAFORMAT_DV2:
-     /* FIXME: not implemented, do default: */
+      dv_parse_header(decoder, jpeg_buffer);
+      switch(decoder->sampling) {
+      /* FIXME: not implemented, do default: */
+      case e_dv_sample_420:
+	  break;
+      case e_dv_sample_411:
+      case e_dv_sample_422:
+         /* libdv decodes NTSC DV (native 411) and by default also PAL
+          * DV (native 420) as packed YUV 422 (YUY2 or 4CC 0x32595559)
+          * where the U and V information is repeated.  This can be
+          * transformed to planar 420 (YV12 or 4CC 0x32315659).
+          * For NTSC DV this transformation is lossy.
+          */
+         pitches[0] = decoder->width * 2;
+         pitches[1] = 0;
+         pitches[2] = 0;
+
+	 dv_decode_full_frame(decoder, jpeg_buffer, e_dv_color_yuv,
+		dv_frame, pitches);
+
+	 output[0] = settings->yuv_overlay->pixels[0];
+	 output[1] = settings->yuv_overlay->pixels[2];
+	 output[2] = settings->yuv_overlay->pixels[1];
+	 frame_YUV422_to_YUV420P(output, dv_frame[0],
+		decoder->width,	decoder->height);
+	 break;
+     }
+     break;
+	
 # endif
    default:
      return 0;
    }
-#endif
 
    if (!lavplay_SDL_unlock(info)) return 0;
    SDL_DisplayYUVOverlay(settings->yuv_overlay, &(settings->jpegdims));
-
    return 1;
 }
-#endif
+#endif /* HAVE_SDL */
 
 
 /******************************************************
@@ -764,9 +905,7 @@ static void *lavplay_mjpeg_playback_thread(void * arg)
       /* There is one buffer to play - get ready to rock ! */
       if (settings->currently_processed_entry != settings->buffer_entry[settings->currently_processed_frame] &&
 	  !lavplay_SDL_update(info, settings->buff+settings->currently_processed_frame*settings->br.size,
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
          settings->data_format[settings->currently_processed_frame],
-#endif
          settings->br.size))
       {
          /* something went wrong - give a warning (don't exit yet) */
@@ -828,7 +967,7 @@ static int lavplay_mjpeg_open(lavplay_t *info)
          if ((settings->video_fd = open(info->video_dev, O_RDWR)) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error opening %s: %s", info->video_dev, sys_errlist[errno]);
+               "Error opening %s: %s", info->video_dev, strerror(errno));
             return 0;
          }
 
@@ -839,7 +978,7 @@ static int lavplay_mjpeg_open(lavplay_t *info)
          if (ioctl(settings->video_fd, MJPIOC_REQBUFS, &(settings->br)) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error requesting buffers: %s", sys_errlist[errno]);
+               "Error requesting buffers: %s", strerror(errno));
             return 0;
          }
 
@@ -849,7 +988,7 @@ static int lavplay_mjpeg_open(lavplay_t *info)
          if (settings->buff == MAP_FAILED)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error mapping the video buffer: %s", sys_errlist[errno]);
+               "Error mapping the video buffer: %s", strerror(errno));
             return 0;
          }
          break;
@@ -931,7 +1070,7 @@ static int lavplay_mjpeg_get_params(lavplay_t *info, struct mjpeg_params *bp)
          if (ioctl(settings->video_fd, MJPIOC_G_PARAMS, bp) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error getting video parameters: %s", sys_errlist[errno]);
+               "Error getting video parameters: %s", strerror(errno));
             return 0;
          }
          break;
@@ -993,7 +1132,7 @@ static int lavplay_mjpeg_set_params(lavplay_t *info, struct mjpeg_params *bp)
       {
          lavplay_msg(LAVPLAY_MSG_ERROR, info,
             "Error getting device capabilities: %s",
-            sys_errlist[errno]);
+            strerror(errno));
          return 0;
       }
       /* hack for wrong return value of marvel cards..... */
@@ -1058,14 +1197,14 @@ static int lavplay_mjpeg_set_params(lavplay_t *info, struct mjpeg_params *bp)
       {
          lavplay_msg(LAVPLAY_MSG_ERROR, info,
             "Could not set on-screen window parameters (check framebuffer-params with v4l-conf): %s",
-            sys_errlist[errno]);
+            strerror(errno));
          return 0;
       }
       n = 1;
       if (ioctl(settings->video_fd, VIDIOCCAPTURE, &n) < 0)
       {
          lavplay_msg(LAVPLAY_MSG_ERROR, info,
-            "Could not activate on-screen window: %s", sys_errlist[errno]);
+            "Could not activate on-screen window: %s", strerror(errno));
          return 0;
       }
 
@@ -1087,7 +1226,7 @@ static int lavplay_mjpeg_set_params(lavplay_t *info, struct mjpeg_params *bp)
          if (ioctl(settings->video_fd, MJPIOC_S_PARAMS, bp) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error setting video parameters: %s", sys_errlist[errno]);
+               "Error setting video parameters: %s", strerror(errno));
             return 0;
          }
          break;
@@ -1148,7 +1287,8 @@ static int lavplay_mjpeg_set_playback_rate(lavplay_t *info, double video_fps, in
       abs(target_usec_per_frame - norm_usec_per_frame) > 50)
    {
       lavplay_msg(LAVPLAY_MSG_ERROR, info,
-         "Specified frame-rate doesn't match in mode in hardware playback");
+         "Specified frame-rate doesn't match in mode in hardware playback (target: %d, norm: %d)",
+		target_usec_per_frame, norm_usec_per_frame);
       return 0;
    }
 
@@ -1178,7 +1318,7 @@ static int lavplay_mjpeg_queue_buf(lavplay_t *info, int frame, int frame_periods
          if (ioctl(settings->video_fd, MJPIOC_QBUF_PLAY, &frame) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error queueing buffer: %s", sys_errlist[errno]);
+               "Error queueing buffer: %s", strerror(errno));
             return 0;
          }
          break;
@@ -1222,7 +1362,7 @@ static int lavplay_mjpeg_sync_buf(lavplay_t *info, struct mjpeg_sync *bs)
          if (ioctl(settings->video_fd, MJPIOC_SYNC, bs) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error syncing on a buffer: %s", sys_errlist[errno]);
+               "Error syncing on a buffer: %s", strerror(errno));
             return 0;
          }
          lavplay_msg(LAVPLAY_MSG_DEBUG, info,
@@ -1281,7 +1421,7 @@ static int lavplay_mjpeg_close(lavplay_t *info)
          if (ioctl(settings->video_fd, MJPIOC_QBUF_PLAY, &n) < 0)
          {
             lavplay_msg(LAVPLAY_MSG_ERROR, info,
-               "Error de-queueing the buffers: %s", sys_errlist[errno]);
+               "Error de-queueing the buffers: %s", strerror(errno));
             return 0;
          }
          if (info->playback_mode == 'H')
@@ -1290,7 +1430,7 @@ static int lavplay_mjpeg_close(lavplay_t *info)
             if (ioctl(settings->video_fd, VIDIOCCAPTURE, &n) < 0)
             {
                lavplay_msg(LAVPLAY_MSG_ERROR, info,
-                  "Could not deactivate on-screen window: %s", sys_errlist[errno]);
+                  "Could not deactivate on-screen window: %s", strerror(errno));
                return 0;
             }
          }
@@ -1401,6 +1541,15 @@ static int lavplay_init(lavplay_t *info)
    }
 #endif
 
+#ifdef	HAVE_LIBDV
+   decoder = dv_decoder_new(0,0,0);
+   decoder->quality = DV_QUALITY_BEST;
+   dv_frame[0] = (uint8_t *)malloc(3 * /*param->output_width * param->output_height */ 720 * 480 * 4);
+   dv_frame[1] = NULL;
+   dv_frame[2] = NULL;
+#endif
+
+
    if (editlist->has_audio && info->audio)
    {
       if (audio_init(0, 0, (editlist->audio_chans>1), editlist->audio_bits,
@@ -1420,7 +1569,7 @@ static int lavplay_init(lavplay_t *info)
    if(seteuid(getuid()) < 0)
    {
       lavplay_msg(LAVPLAY_MSG_ERROR, info,
-         "Can't set effective user-id: %s", sys_errlist[errno]);
+         "Can't set effective user-id: %s", strerror(errno));
       return 0;
    }
 
@@ -1428,10 +1577,8 @@ static int lavplay_init(lavplay_t *info)
    for(nqueue=0;nqueue<settings->br.count;nqueue++)
    {
       if (!lavplay_queue_next_frame(info, settings->buff+nqueue* settings->br.size,
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
 				    (settings->data_format[nqueue] =
 				     el_video_frame_data_format(settings->current_frame_num, editlist)),
-#endif
 				    0,0,0)) break;
    }
 
@@ -1455,7 +1602,7 @@ static int lavplay_init(lavplay_t *info)
       if (ioctl(settings->video_fd, VIDIOCGCAP, &vc) < 0)
       {
          lavplay_msg(LAVPLAY_MSG_ERROR, info,
-            "Error getting device capabilities: %s", sys_errlist[errno]);
+            "Error getting device capabilities: %s", strerror(errno));
          return 0;
       }
       /* vc.maxwidth is often reported wrong - let's just keep it broken (sigh) */
@@ -1582,7 +1729,8 @@ static int lavplay_init(lavplay_t *info)
    if (!lavplay_mjpeg_set_params(info, &bp))
       return 0;
 
-   if (!lavplay_mjpeg_set_playback_rate(info, editlist->video_fps, bp.norm))
+   if (!lavplay_mjpeg_set_playback_rate(info, editlist->video_fps,
+       editlist->video_norm=='p'?0:1))
       return 0;
 
    return 1;
@@ -1729,10 +1877,8 @@ static void lavplay_playback_cycle(lavplay_t *info)
 	 settings->buffer_entry[frame] = editlist->frame_list[settings->current_frame_num];
 #endif
          if (!lavplay_queue_next_frame(info, settings->buff+frame*settings->br.size,
-#if defined(SUPPORT_READ_DV2) || defined(SUPPORT_READ_YUV420)
 				       (settings->data_format[frame] =
 					el_video_frame_data_format(settings->current_frame_num, editlist)),
-#endif
 				       skipv,skipa,skipi))
          {
             lavplay_change_state(info, LAVPLAY_STATE_STOP);
