@@ -50,8 +50,23 @@
 #include <sys/vfs.h>
 #include <stdlib.h>
 
+/* Because of some really cool feature in video4linux1, also known as
+ * 'not including sys/types.h and sys/time.h', we had to include it
+ * ourselves. In all their intelligence, these people decided to fix
+ * this in the next version (video4linux2) in such a cool way that it
+ * breaks all compilations of old stuff...
+ * The real problem is actually that linux/time.h doesn't use proper
+ * macro checks before defining types like struct timeval. The proper
+ * fix here is to either fuck the kernel header (which is what we do
+ * by defining _LINUX_TIME_H, an innocent little hack) or by fixing it
+ * upstream, which I'll consider doing later on. If you get compiler
+ * errors here, check your linux/time.h && sys/time.h header setup.
+ */
+#define _LINUX_TIME_H
 #include <linux/videodev.h>
+#ifdef HAVE_SYS_SOUNDCARD_H
 #include <sys/soundcard.h>
+#endif
 
 #include <videodev_mjpeg.h>
 #include <pthread.h>
@@ -146,6 +161,7 @@ typedef struct {
    pthread_t software_sync_thread;            /* the thread */
    pthread_mutex_t software_sync_mutex;       /* the mutex */
    sig_atomic_t please_stop_syncing;
+   unsigned long buffers_queued;		/* evil hack for BTTV-0.8 */
    int software_sync_ready[MJPEG_MAX_BUF];    /* whether the frame has already been synced on */
    pthread_cond_t software_sync_wait[MJPEG_MAX_BUF]; /* wait for frame to be synced on */
    struct timeval software_sync_timestamp[MJPEG_MAX_BUF];
@@ -306,6 +322,21 @@ static int lavrec_set_mixer(lavrec_t *info, int flag)
          sound_mixer_write_input = SOUND_MIXER_WRITE_LINE;
          sound_mask_input        = SOUND_MASK_LINE;
          break;
+      case '1':
+         sound_mixer_read_input  = SOUND_MIXER_READ_LINE1;
+         sound_mixer_write_input = SOUND_MIXER_WRITE_LINE1;
+         sound_mask_input        = SOUND_MASK_LINE1;
+         break;
+      case '2':
+         sound_mixer_read_input  = SOUND_MIXER_READ_LINE2;
+         sound_mixer_write_input = SOUND_MIXER_WRITE_LINE2;
+         sound_mask_input        = SOUND_MASK_LINE2;
+         break;
+      case '3':
+         sound_mixer_read_input  = SOUND_MIXER_READ_LINE3;
+         sound_mixer_write_input = SOUND_MIXER_WRITE_LINE3;
+         sound_mask_input        = SOUND_MASK_LINE3;
+         break;
       default:
          lavrec_msg(LAVREC_MSG_WARNING, info,
             "Unknown sound source: \'%c\'", info->audio_src);
@@ -418,7 +449,7 @@ static int lavrec_autodetect_signal(lavrec_t *info)
          {
             lavrec_msg (LAVREC_MSG_ERROR, info,
                "Error getting video input status: %s",
-               (const char*)sys_errlist[errno]);
+               (const char*)strerror(errno));
             return 0;
          }
 
@@ -469,7 +500,7 @@ static int lavrec_autodetect_signal(lavrec_t *info)
       if (ioctl(settings->video_fd,MJPIOC_G_STATUS,&bstat) < 0)
       {
          lavrec_msg (LAVREC_MSG_ERROR, info,
-            "Error getting video input status: %s",sys_errlist[errno]);
+            "Error getting video input status: %s",strerror(errno));
          return 0;
       }
 
@@ -903,6 +934,10 @@ static void *lavrec_encoding_thread(void* arg)
             current_frame);
          pthread_cond_wait(&(settings->buffer_filled[current_frame]),
             &(settings->encoding_mutex));
+         if (settings->please_stop_syncing) {
+            pthread_mutex_unlock(&(settings->encoding_mutex));
+            pthread_exit(NULL);
+         }
       }
       memcpy(&(timestamp[current_frame]), &(settings->bsync.timestamp), sizeof(struct timeval));
 
@@ -966,18 +1001,21 @@ static void *lavrec_encoding_thread(void* arg)
          }
       }
 
-
+#if 0
       if (!lavrec_queue_buffer(info, &current_frame))
       {
          if (info->files)
             lavrec_close_files_on_error(info);
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error re-queuing buffer: %s", sys_errlist[errno]);
+            "Error re-queuing buffer: %s", strerror(errno));
          lavrec_change_state(info, LAVREC_STATE_STOP);
          pthread_exit(0);
       }
       /* Mark the capture buffer as once again as in progress for capture */
       settings->buffer_valid[current_frame] = -1;
+#endif
+      /* hack for BTTV-0.8 - give it a status that tells us to queue it in another thread */
+      settings->buffer_valid[current_frame] = -2;
 
       if (!lavrec_handle_audio(info, &(timestamp[current_frame])))
          lavrec_change_state(info, LAVREC_STATE_STOP);
@@ -1014,7 +1052,7 @@ static int lavrec_software_init(lavrec_t *info)
    if (ioctl(settings->video_fd, VIDIOCGCAP, &vc) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error getting device capabilities: %s", sys_errlist[errno]);
+         "Error getting device capabilities: %s", strerror(errno));
       return 0;
    }
    /* vc.maxwidth is often reported wrong - let's just keep it broken (sigh) */
@@ -1077,7 +1115,7 @@ static int lavrec_software_init(lavrec_t *info)
    if (ioctl(settings->video_fd, VIDIOCGMBUF, &(settings->softreq)) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error getting buffer information: %s", sys_errlist[errno]);
+         "Error getting buffer information: %s", strerror(errno));
       return 0;
    }
    if (settings->softreq.frames < MIN_QUEUES_NEEDED)
@@ -1097,7 +1135,7 @@ static int lavrec_software_init(lavrec_t *info)
    if (settings->YUV_buff == MAP_FAILED)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error mapping video buffers: %s", sys_errlist[errno]);
+         "Error mapping video buffers: %s", strerror(errno));
       return 0;
    }
 
@@ -1167,7 +1205,7 @@ static int lavrec_hardware_init(lavrec_t *info)
    if (ioctl(settings->video_fd, VIDIOCGCAP, &vc) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error getting device capabilities: %s", sys_errlist[errno]);
+         "Error getting device capabilities: %s", strerror(errno));
       return 0;
    }
    /* vc.maxwidth is often reported wrong - let's just keep it broken (sigh) */
@@ -1177,7 +1215,7 @@ static int lavrec_hardware_init(lavrec_t *info)
    if (ioctl(settings->video_fd, MJPIOC_G_PARAMS, &bparm) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error getting video parameters: %s", sys_errlist[errno]);
+         "Error getting video parameters: %s", strerror(errno));
       return 0;
    }
    bparm.input = info->video_src;
@@ -1277,7 +1315,7 @@ static int lavrec_hardware_init(lavrec_t *info)
    if (ioctl(settings->video_fd, MJPIOC_S_PARAMS, &bparm) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error setting video parameters: %s", sys_errlist[errno]);
+         "Error setting video parameters: %s", strerror(errno));
       return 0;
    }
 
@@ -1295,7 +1333,7 @@ static int lavrec_hardware_init(lavrec_t *info)
    if (ioctl(settings->video_fd, MJPIOC_REQBUFS,&(settings->breq)) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error requesting video buffers: %s", sys_errlist[errno]);
+         "Error requesting video buffers: %s", strerror(errno));
       return 0;
    }
    lavrec_msg(LAVREC_MSG_INFO, info,
@@ -1307,7 +1345,7 @@ static int lavrec_hardware_init(lavrec_t *info)
    if (settings->MJPG_buff == MAP_FAILED)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error mapping video buffers: %s", sys_errlist[errno]);
+         "Error mapping video buffers: %s", strerror(errno));
       return 0;
    }
 
@@ -1401,7 +1439,7 @@ static int lavrec_init(lavrec_t *info)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
             "Failed to set effective user-ID: %s",
-            sys_errlist[errno]);
+            strerror(errno));
          return 0;
       }
    }
@@ -1412,7 +1450,7 @@ static int lavrec_init(lavrec_t *info)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
          "Error opening video-device (%s): %s",
-         info->video_dev, sys_errlist[errno]);
+         info->video_dev, strerror(errno));
       return 0;
    }
 
@@ -1430,14 +1468,14 @@ static int lavrec_init(lavrec_t *info)
       if (ioctl(settings->video_fd, VIDIOCSCHAN, &vch) < 0)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error setting channel: %s", sys_errlist[errno]);
+            "Error setting channel: %s", strerror(errno));
          return 0;
       }
    }
    if (ioctl(settings->video_fd, VIDIOCGCHAN, &vch) < 0)
    {
       lavrec_msg(LAVREC_MSG_ERROR, info,
-         "Error getting channel info: %s", sys_errlist[errno]);
+         "Error getting channel info: %s", strerror(errno));
       return 0;
    }
    settings->has_audio = (vch.flags & VIDEO_VC_AUDIO);
@@ -1451,7 +1489,7 @@ static int lavrec_init(lavrec_t *info)
       if (ioctl(settings->video_fd, VIDIOCSFREQ, &outfreq) < 0)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error setting tuner frequency: %s", sys_errlist[errno]);
+            "Error setting tuner frequency: %s", strerror(errno));
          return 0;
       }
    }
@@ -1467,7 +1505,7 @@ static int lavrec_init(lavrec_t *info)
       if (ioctl(settings->video_fd,VIDIOCGAUDIO, &vau) < 0)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error getting tuner audio params: %s", sys_errlist[errno]);
+            "Error getting tuner audio params: %s", strerror(errno));
          return 0;
       }
       /* unmute so we get sound to record
@@ -1479,7 +1517,7 @@ static int lavrec_init(lavrec_t *info)
       if (ioctl(settings->video_fd,VIDIOCSAUDIO, &vau) < 0)
       {
          lavrec_msg(LAVREC_MSG_INFO, info,
-            "Error setting tuner audio params: %s", sys_errlist[errno]);
+            "Error setting tuner audio params: %s", strerror(errno));
          return 0;
       }
    }
@@ -1609,6 +1647,7 @@ static int lavrec_queue_buffer(lavrec_t *info, unsigned long *num)
       pthread_mutex_lock(&(settings->queue_mutex));
       settings->queue_left++;
       settings->is_queued[*num] = 1;
+      settings->buffers_queued++;
       pthread_cond_broadcast(&(settings->queue_wait));
       pthread_mutex_unlock(&(settings->queue_mutex));
    }
@@ -1632,6 +1671,9 @@ static void *lavrec_software_sync_thread(void* arg)
    lavrec_t *info = (lavrec_t *) arg;
    video_capture_setup *settings = (video_capture_setup *)info->settings;
    int frame = 0; /* framenum to sync on */
+#if 1
+   unsigned long qframe, i;
+#endif
 
    /* Allow easy shutting down by other processes... */
    /* PTHREAD_CANCEL_ASYNCHRONOUS is evil
@@ -1647,25 +1689,82 @@ static void *lavrec_software_sync_thread(void* arg)
 
    while (1)
    {
-      pthread_mutex_lock(&(settings->queue_mutex));
+      /* evil hack for BTTV-0.8 - we need to queue frames here */
+      /* this cycle is non-onbligatory - we just queue frames as they become available,
+       * below, we'll wait for queues if we don't have enough of them */
+      for (i=0;i<settings->softreq.frames;i++)
+      {
+         qframe = settings->buffers_queued % settings->softreq.frames;
+         if (settings->buffer_valid[qframe] == -2)
+         {
+            if (!lavrec_queue_buffer(info, &qframe))
+            {
+               pthread_mutex_lock(&(settings->software_sync_mutex));
+               settings->software_sync_ready[qframe] = -1;
+               pthread_cond_broadcast(&(settings->software_sync_wait[qframe]));
+               pthread_mutex_unlock(&(settings->software_sync_mutex));
+               lavrec_msg(LAVREC_MSG_ERROR, info,
+                  "Error re-queueing a buffer (%lu): %s", qframe, strerror(errno));
+               lavrec_change_state(info, LAVREC_STATE_STOP);
+               pthread_exit(0);
+            }
+            settings->buffer_valid[qframe] = -1;
+         }
+         else
+            break;
+      }
+
+      pthread_mutex_lock(&(settings->encoding_mutex));
       while (settings->queue_left < MIN_QUEUES_NEEDED)
       {
 	 if (settings->is_queued[frame] <= 0 ||
              settings->please_stop_syncing)
             break; /* sync on all remaining frames */
+#if 0
          lavrec_msg(LAVREC_MSG_DEBUG, info,
             "Software sync thread: sleeping for new queues (%d)", frame);
          pthread_cond_wait(&(settings->queue_wait),
             &(settings->queue_mutex));
+#else
+         /* sleep for new buffers to be completed encoding. After that,
+          * requeue them so we have more than MIN_QUEUES_NEEDED buffers
+          * free */
+         qframe = settings->buffers_queued % settings->softreq.frames;
+         lavrec_msg(LAVREC_MSG_DEBUG, info,
+            "Software sync thread: sleeping for new queues (%lu) to become available", qframe);
+         while (settings->buffer_valid[qframe] != -2)
+         {
+            pthread_cond_wait(&(settings->buffer_completion[qframe]),
+               &(settings->encoding_mutex));
+            if (settings->please_stop_syncing) {
+               pthread_mutex_unlock(&(settings->encoding_mutex));
+               pthread_exit(0);
+            }
+         }
+         if (!lavrec_queue_buffer(info, &qframe))
+         {
+            pthread_mutex_unlock(&(settings->encoding_mutex));
+            pthread_mutex_lock(&(settings->software_sync_mutex));
+            settings->software_sync_ready[qframe] = -1;
+            pthread_cond_broadcast(&(settings->software_sync_wait[qframe]));
+            pthread_mutex_unlock(&(settings->software_sync_mutex));
+            lavrec_msg(LAVREC_MSG_ERROR, info,
+               "Error re-queueing a buffer (%lu): %s", qframe, strerror(errno));
+            lavrec_change_state(info, LAVREC_STATE_STOP);
+            pthread_exit(0);
+         }
+         settings->buffer_valid[qframe] = -1;
+#endif
       }
+
       if (!settings->queue_left)
       {
 	 lavrec_msg(LAVREC_MSG_DEBUG, info,
 		    "Software sync thread stopped");
-	 pthread_mutex_unlock(&settings->queue_mutex);
+	 pthread_mutex_unlock(&settings->encoding_mutex);
 	 pthread_exit(NULL);
       }
-      pthread_mutex_unlock(&settings->queue_mutex);
+      pthread_mutex_unlock(&settings->encoding_mutex);
       
 retry:
       if (ioctl(settings->video_fd, VIDIOCSYNC, &frame) < 0)
@@ -1676,7 +1775,7 @@ retry:
          pthread_cond_broadcast(&(settings->software_sync_wait[frame]));
          pthread_mutex_unlock(&(settings->software_sync_mutex));
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error syncing on a buffer: %s", sys_errlist[errno]);
+            "Error syncing on a buffer: %s", strerror(errno));
          lavrec_change_state(info, LAVREC_STATE_STOP);
          pthread_exit(0);
       }
@@ -1861,10 +1960,12 @@ static void lavrec_record(lavrec_t *info)
       for (x=0;x<MJPEG_MAX_BUF;x++)
       {
          settings->is_queued[x] = 0;
-         settings->buffer_valid[x] = -1; /* 0 means to just omit the frame, -1 means "in progress" */
+         settings->buffer_valid[x] = -1; /* 0 means to just omit the frame, -1 means "in progress",
+						-2 is an evil hack for BTTV-0.8 */
          settings->buffer_completed[x] = 1; /* 1 means compression and writing completed,
                                                0 means in progress */
       }
+      settings->buffers_queued = 0;
 
       if( !(settings->encoders = malloc(info->num_encoders * sizeof(encoder_info_t))) )
       {
@@ -1901,7 +2002,7 @@ static void lavrec_record(lavrec_t *info)
       if (!lavrec_queue_buffer(info, &frame_cnt))
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error queuing buffers: %s", sys_errlist[errno]);
+            "Error queuing buffers: %s", strerror(errno));
          lavrec_change_state(info, LAVREC_STATE_STOP);
          return;
       }
@@ -1954,7 +2055,7 @@ static void lavrec_record(lavrec_t *info)
          if (info->files)
             lavrec_close_files_on_error(info);
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error syncing on a buffer: %s", sys_errlist[errno]);
+            "Error syncing on a buffer: %s", strerror(errno));
          nerr++;
       }
       stats.num_syncs++;
@@ -2046,7 +2147,7 @@ static void lavrec_record(lavrec_t *info)
             if (info->files)
                lavrec_close_files_on_error(info);
             lavrec_msg(LAVREC_MSG_ERROR, info,
-               "Error re-queuing buffer: %s", sys_errlist[errno]);
+               "Error re-queuing buffer: %s", strerror(errno));
             nerr++;
          }
 
@@ -2060,10 +2161,11 @@ static void lavrec_record(lavrec_t *info)
 
    if (info->software_encoding)
    {
-      pthread_mutex_lock(&settings->queue_mutex);
+      pthread_mutex_lock(&settings->encoding_mutex);
       settings->please_stop_syncing = 1; /* Ask the software sync thread to stop */
-      pthread_cond_broadcast(&settings->queue_wait);
-      pthread_mutex_unlock(&settings->queue_mutex);
+      for (x=0;x<settings->softreq.frames;x++)
+        pthread_cond_broadcast(&settings->buffer_completion[x]);
+      pthread_mutex_unlock(&settings->encoding_mutex);
       
       for (x = 0; x < info->num_encoders; x++)
       {
@@ -2091,7 +2193,7 @@ static void lavrec_record(lavrec_t *info)
       if (ioctl(settings->video_fd, MJPIOC_QBUF_CAPT, &x) < 0)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error resetting buffer-queue: %s", sys_errlist[errno]);
+            "Error resetting buffer-queue: %s", strerror(errno));
       }
    }
 }
@@ -2160,7 +2262,7 @@ static void *lavrec_capture_thread(void *arg)
       if (ioctl(settings->video_fd,VIDIOCSAUDIO,&vau) < 0)
       {
          lavrec_msg(LAVREC_MSG_ERROR, info,
-            "Error resetting tuner audio params: %s", sys_errlist[errno]);
+            "Error resetting tuner audio params: %s", strerror(errno));
       }
    }
 
@@ -2296,6 +2398,13 @@ int lavrec_main(lavrec_t *info)
    int ret;
    struct sched_param schedparam;
 
+   /* Flush the Linux File buffers to disk */
+   sync();
+
+   /* start with initing */
+   if (!lavrec_init(info))
+      return 0;
+
    /* Now we're ready to go move to Real-time scheduling... */
    schedparam.sched_priority = 1;
    if(setpriority(PRIO_PROCESS, 0, -15)) { /* Give myself maximum priority */ 
@@ -2307,13 +2416,6 @@ int lavrec_main(lavrec_t *info)
          "Pthread Real-time scheduling for main thread could not be enabled"); 
    }
 
-   /* Flush the Linux File buffers to disk */
-   sync();
-
-   /* start with initing */
-   if (!lavrec_init(info))
-      return 0;
-
    /* now, set state to pause and catch audio until started */
    /* lavrec_change_state(info, LAVREC_STATE_PAUSED); */
    settings->state = LAVREC_STATE_PAUSED;
