@@ -1,4 +1,4 @@
-/* motion.c, motion estimation                                              */
+/* motionsearch.c, block motion estimation for mpeg2enc  */
 
 
 /* (C) 2000/2001 Andrew Stevens */
@@ -20,76 +20,45 @@
  *
  */
 
-#include <config.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include "config.h"
 #include <limits.h>
-#include "math.h"
+#include <math.h>
 #include "cpu_accel.h"
 #include "fastintfns.h"
 #include "motionsearch.h"
 #include "mjpeg_logging.h"
 
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
-
-#include "mblock_sub44_sads_x86.h"
-
-static int (*pmblocks_sub44_mests)( uint8_t *blk,  uint8_t *ref,
-							int ilow, int jlow,
-							int ihigh, int jhigh, 
-							int h, int rowstride, 
-							int threshold,
-							me_result_s *resvec);
-
-
-void mblock_sub22_nearest4_sads_mmxe(uint8_t *blk1,uint8_t *blk2,
-				     int frowstride,int fh, int* resvec) 
-                                     __asm__ ("mblock_sub22_nearest4_sads_mmxe");
-
-void mblock_nearest4_sads_mmxe(uint8_t *blk1, uint8_t *blk2, 
-			       int rowstride, int h, int *resvec)
-                               __asm__ ("mblock_nearest4_sads_mmxe");
-
-int sad_00_mmxe(uint8_t *blk1, uint8_t *blk2, int rowstride, int h, int distlim) __asm__ ("sad_00_mmxe");
-int sad_01_mmxe(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_01_mmxe");
-int sad_10_mmxe(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_10_mmxe");
-int sad_11_mmxe(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_11_mmxe");
-
-
-int sad_sub22_mmxe ( uint8_t *blk1, uint8_t *blk2,  int frowstride, int fh) __asm__ ("sad_sub22_mmxe");
-int sad_sub44_mmxe ( uint8_t *blk1, uint8_t *blk2,  int qrowstride, int qh) __asm__ ("sad_sub44_mmxe");
-int sumsq_mmx( uint8_t *blk1, uint8_t *blk2,
-	       int rowstride, int hx, int hy, int h) __asm__ ("sumsq_mmx");
-int sumsq_sub22_mmx( uint8_t *blk1, uint8_t *blk2,
-		     int rowstride, int h) __asm__ ("sumsq_sub22_mmx");
-int bsumsq_sub22_mmx( uint8_t *blk1f, uint8_t *blk1b, 
-		      uint8_t *blk2,
-		      int rowstride, int h) __asm__ ("bsumsq_sub22_mmx");
-int bsumsq_mmx (uint8_t *pf, uint8_t *pb,
-		uint8_t *p2, int rowstride,
-		int hxf, int hyf, int hxb, int hyb, int h) __asm__ ("bsumsq_mmx");
-int bsad_mmx (uint8_t *pf, uint8_t *pb,
-	      uint8_t *p2, int rowstride,
-	      int hxf, int hyf, int hxb, int hyb, int h) __asm__ ("bsad_mmx");
-
-int variance_mmx( uint8_t *p, int size,	int rowstride) __asm__ ("variance_mmx");
-
-int sad_00_mmx ( uint8_t *blk1, uint8_t *blk2,  int rowstride, int h, int distlim) __asm__ ("sad_00_mmx");
-int sad_01_mmx(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_01_mmx");
-int sad_10_mmx(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_10_mmx");
-int sad_11_mmx(uint8_t *blk1, uint8_t *blk2, int rowstride, int h) __asm__ ("sad_11_mmx");
-int sad_sub22_mmx ( uint8_t *blk1, uint8_t *blk2,  int frowstride, int fh) __asm__ ("sad_sub22_mmx");
-int sad_sub44_mmx (uint8_t *blk1, uint8_t *blk2,  int qrowstride, int qh) __asm__ ("sad_sub44_mmx");
-
+/* The AltiVec code needs access to symbols during benchmarking
+ * and verification.
+ */
+#define STATIC static
+#ifdef HAVE_ALTIVEC
+#include "altivec/altivec_motion.h"
+#  if ALTIVEC_TEST_MOTION
+#    undef STATIC
+#    define STATIC /* static */
+#  endif
 #endif
 
 
+
+#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
+#include "mmxsse/mmxsse_motion.h"
+#endif
+
 /*
  * Function pointers for selecting CPU specific implementations
  *
  */
 
+int (*pmblocks_sub44_mests)( uint8_t *blk,  uint8_t *ref,
+							 int ilow, int jlow,
+							 int ihigh, int jhigh, 
+							 int h, int rowstride, 
+							 int threshold,
+							 me_result_s *resvec);
+
 void (*pfind_best_one_pel)( me_result_set *sub22set,
 							uint8_t *org, uint8_t *blk,
 							int i0, int j0,
@@ -120,7 +89,8 @@ int (*pbsumsq_sub22)( uint8_t *blk1f, uint8_t *blk1b,
 						  uint8_t *blk2,
 						  int rowstride, int h);
 
-int (*pvariance)(uint8_t *mb, int size, int rowstride);
+void (*pvariance)(uint8_t *mb, int size, int rowstride, 
+				 uint32_t *p_var, uint32_t *p_mean);
 
 
 int (*psad_sub22) ( uint8_t *blk1, uint8_t *blk2,  int frowstride, int fh);
@@ -141,6 +111,9 @@ int (*pbsad) (uint8_t *pf, uint8_t *pb,
 					   uint8_t *p2, int rowstride, int hxf, int hyf, int hxb, int hyb, int h);
 
 
+void (*psubsample_image) (uint8_t *image, int rowstride, 
+						uint8_t *sub22_image, 
+						uint8_t *sub44_image);
 
 
 
@@ -162,9 +135,9 @@ int round_search_radius( int radius )
 	discarding all elements whose sad "weight" is above the current mean weight.
 */
 
-static void sub_mean_reduction( me_result_set *matchset, 
-								int times,
-							    int *minweight_res)
+void sub_mean_reduction( me_result_set *matchset, 
+						 int times,
+						 int *minweight_res)
 {
 	me_result_s *matches = matchset->mests;
 	int len = matchset->len;
@@ -243,7 +216,7 @@ static void sub_mean_reduction( me_result_set *matchset,
  * 
  */
 
-static int build_sub44_mests( me_result_set *sub44set,
+STATIC int build_sub44_mests( me_result_set *sub44set,
 							   int ilow, int jlow, int ihigh, int jhigh, 
 							   int i0, int j0,
 							   int null_ctl_sad,
@@ -312,45 +285,6 @@ static int build_sub44_mests( me_result_set *sub44set,
 	return sub44set->len;
 }
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
-
-static int build_sub44_mests_mmx( me_result_set *sub44set,
-								   int ilow, int jlow, int ihigh, int jhigh, 
-								   int i0, int j0,
-								   int null_ctl_sad,
-								   uint8_t *s44org, uint8_t *s44blk, 
-								   int qrowstride, int qh,
-								   int reduction)
-{
-	uint8_t *s44orgblk;
-	me_result_s *sub44_mests = sub44set->mests;
-	int istrt = ilow-i0;
-	int jstrt = jlow-j0;
-	int iend = ihigh-i0;
-	int jend = jhigh-j0;
-	int mean_weight;
-	int threshold;
-
-	
-	threshold = 6*null_ctl_sad / (4*4*reduction);
-	s44orgblk = s44org+(ilow>>2)+qrowstride*(jlow>>2);
-	
-	sub44set->len = (*pmblocks_sub44_mests)( s44orgblk, s44blk,
-											  istrt, jstrt,
-											  iend, jend, 
-											  qh, qrowstride, 
-											  threshold,
-											  sub44_mests);
-	
-   /* If we're really pushing quality we reduce once otherwise twice. */
-			
-	sub_mean_reduction( sub44set, 1+(reduction>1),  &mean_weight);
-
-
-	return sub44set->len;
-}
-#endif
-
 
 
 /*  Build a vector of the best 2*2 sub-sampled motion estimations for
@@ -368,11 +302,10 @@ static int build_sub44_mests_mmx( me_result_set *sub44set,
  * a reference SAD passed as a parameter and then reduction discard passes
  * are made.
  *
- * A super-fast version using MMX assembly code for X86 follows.
- * Other CPU's could/should be handled the same way.  */
+ */
 
 
-static int build_sub22_mests( me_result_set *sub44set,
+STATIC int build_sub22_mests( me_result_set *sub44set,
 							   me_result_set *sub22set,
 							   int i0,  int j0, int ihigh, int jhigh, 
 							   int null_ctl_sad,
@@ -433,77 +366,6 @@ static int build_sub22_mests( me_result_set *sub44set,
 	return sub22set->len;
 }
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM) 
-static int build_sub22_mests_mmxe( me_result_set *sub44set,
-							 me_result_set *sub22set,
-							 int i0,  int j0, int ihigh, int jhigh, 
-							 int null_ctl_sad,
-							 uint8_t *s22org,  uint8_t *s22blk, 
-							 int frowstride, int fh,
-							 int reduction)
-{
-	int i,k,s;
-	int threshold = 6*null_ctl_sad / (2 * 2*reduction);
-
-	int min_weight;
-	int ilim = ihigh-i0;
-	int jlim = jhigh-j0;
-	int x,y;
-	uint8_t *s22orgblk;
-	int resvec[4];
-
-	/* TODO: The calculation of the lstrow offset really belongs in
-       asm code... */
-	int lstrow=(fh-1)*frowstride;
-
-	sub22set->len = 0;
-	for( k = 0; k < sub44set->len; ++k )
-	{
-
-		x = sub44set->mests[k].x;
-		y = sub44set->mests[k].y;
-
-		s22orgblk =  s22org +((y+j0)>>1)*frowstride +((x+i0)>>1);
-		/*
-		  Get SAD for 2*2 subsampled macroblocks: orgblk,orgblk(+2,0),
-		  orgblk(0,+2), and orgblk(+2,+2) Done all in one go to reduce
-		  memory bandwidth demand
-		*/
-		mblock_sub22_nearest4_sads_mmxe(s22orgblk+lstrow, s22blk+lstrow, frowstride, fh, resvec);
-		for( i = 0; i < 4; ++i )
-		{
-			if( x <= ilim && y <= jlim )
-			{	
-				s =resvec[i]+(intmax(intabs(x),intabs(y))<<3);
-				if( s < threshold )
-				{
-					me_result_s *mc = &sub22set->mests[sub22set->len];
-					mc->x = (int8_t)x;
-					mc->y = (int8_t)y;
-					mc->weight = s;
-					++(sub22set->len);
-				}
-			}
-
-			if( i == 1 )
-			{
-				x -= 2;
-				y += 2;
-			}
-			else
-			{
-				x += 2;
-			}
-		}
-
-	}
-
-	
-	sub_mean_reduction( sub22set, reduction, &min_weight );
-	return sub22set->len;
-}
-
-#endif
 
 /*
  * Search for the best 1-pel match within 1-pel of a good 2*2-pel
@@ -513,7 +375,7 @@ static int build_sub22_mests_mmxe( me_result_set *sub44set,
  */
 
 
-static void find_best_one_pel( me_result_set *sub22set,
+STATIC void find_best_one_pel( me_result_set *sub22set,
 							   uint8_t *org, uint8_t *blk,
 							   int i0, int j0,
 							   int ihigh, int jhigh,
@@ -569,71 +431,6 @@ static void find_best_one_pel( me_result_set *sub22set,
 	*best_so_far = minpos;
 }
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM) 
-static void find_best_one_pel_mmxe( me_result_set *sub22set,
-							 uint8_t *org, uint8_t *blk,
-							 int i0, int j0,
-							 int ihigh, int jhigh,
-							 int rowstride, int h,
-							 me_result_s *best_so_far
-	)
-
-{
-	int i,k;
-	int d;
-	me_result_s minpos = *best_so_far;
-	int ilim = ihigh-i0;
-	int jlim = jhigh-j0;
-	int dmin = INT_MAX;
-	uint8_t *orgblk;
-	int penalty;
-	me_result_s matchrec;
-	int resvec[4];
-
-	for( k = 0; k < sub22set->len; ++k )
-	{	
-		matchrec = sub22set->mests[k];
-		orgblk = org + (i0+matchrec.x)+rowstride*(j0+matchrec.y);
-		penalty = intmax(abs(matchrec.x),abs(matchrec.y))<<5;
-		
-		/* Get SAD for macroblocks: 	orgblk,orgblk(+1,0),
-		   orgblk(0,+1), and orgblk(+1,+1)
-		   Done all in one go to reduce memory bandwidth demand
-		*/
-		mblock_nearest4_sads_mmxe(orgblk,blk,rowstride,h,
-		resvec);
-
-		for( i = 0; i < 4; ++i )
-		{
-			if( matchrec.x <= ilim && matchrec.y <= jlim )
-			{
-		
-				d = penalty+resvec[i];
-				if (d<dmin)
-				{
-					dmin = d;
-					minpos = matchrec;
-				}
-			}
-			if( i == 1 )
-			{
-				orgblk += rowstride-1;
-				matchrec.x -= 1;
-				matchrec.y += 1;
-			}
-			else
-			{
-				orgblk += 1;
-				matchrec.x += 1;
-			}
-		}
-	}
-
-	minpos.weight = (uint16_t)intmin(255*255, dmin);
-	*best_so_far = minpos;
-
-}
-#endif 
  
 
 /* 
@@ -649,7 +446,7 @@ static void find_best_one_pel_mmxe( me_result_set *sub22set,
  **/
 
 
-static int sad_00(uint8_t *blk1,uint8_t *blk2,
+STATIC int sad_00(uint8_t *blk1,uint8_t *blk2,
 					int rowstride, int h,int distlim)
 {
 	uint8_t *p1,*p2;
@@ -679,7 +476,7 @@ static int sad_00(uint8_t *blk1,uint8_t *blk2,
 	return s;
 }
 
-static int sad_01(uint8_t *blk1,uint8_t *blk2,int rowstride, int h)
+STATIC int sad_01(uint8_t *blk1,uint8_t *blk2,int rowstride, int h)
 {
 	uint8_t *p1,*p2;
 	int i,j;
@@ -703,7 +500,7 @@ static int sad_01(uint8_t *blk1,uint8_t *blk2,int rowstride, int h)
 	return s;
 }
 
-static int sad_10(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
+STATIC int sad_10(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
 {
 	uint8_t *p1,*p1a,*p2;
 	int i,j;
@@ -729,7 +526,7 @@ static int sad_10(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
 	return s;
 }
 
-static int sad_11(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
+STATIC int sad_11(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
 {
 	uint8_t *p1,*p1a,*p2;
 	int i,j;
@@ -828,7 +625,7 @@ void subsample_image( uint8_t *image, int rowstride,
  */
  
 
-static int sad_sub22( uint8_t *s22blk1, uint8_t *s22blk2,int frowstride,int fh)
+STATIC int sad_sub22( uint8_t *s22blk1, uint8_t *s22blk2,int frowstride,int fh)
 {
 	uint8_t *p1 = s22blk1;
 	uint8_t *p2 = s22blk2;
@@ -863,7 +660,7 @@ static int sad_sub22( uint8_t *s22blk1, uint8_t *s22blk2,int frowstride,int fh)
  */
 
 
-static int sad_sub44( uint8_t *s44blk1, uint8_t *s44blk2,int qrowstride,int qh)
+STATIC int sad_sub44( uint8_t *s44blk1, uint8_t *s44blk2,int qrowstride,int qh)
 {
 	register uint8_t *p1 = s44blk1;
 	register uint8_t *p2 = s44blk2;
@@ -897,7 +694,7 @@ static int sad_sub44( uint8_t *s44blk1, uint8_t *s44blk2,int qrowstride,int qh)
  * h:         height of block (usually 8 or 16)
  */
  
-static int sumsq_sub22(uint8_t *blk1, uint8_t *blk2, int rowstride, int h)
+STATIC int sumsq_sub22(uint8_t *blk1, uint8_t *blk2, int rowstride, int h)
 {
 	uint8_t *p1 = blk1, *p2 = blk2;
 	int i,j,v;
@@ -924,7 +721,7 @@ static int sumsq_sub22(uint8_t *blk1, uint8_t *blk2, int rowstride, int h)
  * h: height of block (usually 4 or 8)
  */
  
-static int bsumsq_sub22(uint8_t *blk1f, uint8_t *blk1b, uint8_t *blk2, 
+STATIC int bsumsq_sub22(uint8_t *blk1f, uint8_t *blk1b, uint8_t *blk2, 
 					 int rowstride, int h)
 {
 	uint8_t *p1f = blk1f,*p1b = blk1b,*p2 = blk2;
@@ -954,7 +751,7 @@ static int bsumsq_sub22(uint8_t *blk1f, uint8_t *blk1b, uint8_t *blk2,
  */
  
 
-static int sumsq(blk1,blk2,rowstride,hx,hy,h)
+STATIC int sumsq(blk1,blk2,rowstride,hx,hy,h)
 	uint8_t *blk1,*blk2;
 	int rowstride,hx,hy,h;
 {
@@ -1034,7 +831,7 @@ static int sumsq(blk1,blk2,rowstride,hx,hy,h)
  */
  
 
-static int bsad(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
+STATIC int bsad(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
 	uint8_t *pf,*pb,*p2;
 	int rowstride,hxf,hyf,hxb,hyb,h;
 {
@@ -1087,7 +884,7 @@ static int bsad(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
  */
  
 
-static int bsumsq(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
+STATIC int bsumsq(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
 	uint8_t *pf,*pb,*p2;
 	int rowstride,hxf,hyf,hxb,hyb,h;
 {
@@ -1138,11 +935,11 @@ static int bsumsq(pf,pb,p2,rowstride,hxf,hyf,hxb,hyb,h)
  * rowstride: distance (in bytes) of vertically adjacent pels
  * SIZE is a multiple of 8.
  */
-static int variance(uint8_t *p, int size,	int rowstride)
+void variance(uint8_t *p, int size,	int rowstride,
+			 unsigned int *p_var, unsigned int *p_mean)
 {
 	int i,j;
 	unsigned int v,s,s2;
-	int var;
 	s = s2 = 0;
 
 	for (j=0; j<size; j++)
@@ -1155,9 +952,8 @@ static int variance(uint8_t *p, int size,	int rowstride)
 		}
 		p+= rowstride-size;
 	}
-	var = s2 - (s*s)/(size*size);
-
-	return var;
+	*p_mean = s/(size*size);
+	*p_var = s2 - (s*s)/(size*size);
 }
 
 
@@ -1173,65 +969,32 @@ void init_motion_search(void)
 {
 	int cpucap = cpu_accel();
 
-	if( cpucap == 0 )	/* No MMX/SSE etc support available */
-	{
-		psad_sub22 = sad_sub22;
-		psad_sub44 = sad_sub44;
-		psad_00 = sad_00;
-		psad_01 = sad_01;
-		psad_10 = sad_10;
-		psad_11 = sad_11;
-		pbsad = bsad;
-		pvariance = variance;
-		psumsq = sumsq;
-		pbsumsq = bsumsq;
-		psumsq_sub22 = sumsq_sub22;
-		pbsumsq_sub22 = bsumsq_sub22;
-		pfind_best_one_pel = find_best_one_pel;
-		pbuild_sub22_mests	= build_sub22_mests;
-		pbuild_sub44_mests	= build_sub44_mests;
-	 }
+	/* Initialize function pointers. This allows partial acceleration
+	 * implementations to update only the function pointers they support.
+	 */
+	psad_sub22 = sad_sub22;
+	psad_sub44 = sad_sub44;
+	psad_00 = sad_00;
+	psad_01 = sad_01;
+	psad_10 = sad_10;
+	psad_11 = sad_11;
+	pbsad = bsad;
+	pvariance = variance;
+	psumsq = sumsq;
+	pbsumsq = bsumsq;
+	psumsq_sub22 = sumsq_sub22;
+	pbsumsq_sub22 = bsumsq_sub22;
+	pfind_best_one_pel = find_best_one_pel;
+	pbuild_sub22_mests = build_sub22_mests;
+	pbuild_sub44_mests = build_sub44_mests;
+	psubsample_image = subsample_image;
+
 #if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
-	else if(cpucap & ACCEL_X86_MMXEXT ) /* AMD MMX or SSE... */
-	{
-		mjpeg_info( "SETTING EXTENDED MMX for MOTION!");
-		psad_sub22 = sad_sub22_mmxe;
-		psad_sub44 = sad_sub44_mmxe;
-		psad_00 = sad_00_mmxe;
-		psad_01 = sad_01_mmxe;
-		psad_10 = sad_10_mmxe;
-		psad_11 = sad_11_mmxe;
-		pbsad = bsad_mmx;
-		pvariance = variance_mmx;
-		psumsq = sumsq_mmx;
-		pbsumsq = bsumsq_mmx;
-		psumsq_sub22 = sumsq_sub22_mmx;
-		pbsumsq_sub22 = bsumsq_sub22_mmx;
-		pfind_best_one_pel = find_best_one_pel_mmxe;
-		pbuild_sub22_mests	= build_sub22_mests_mmxe;
-		pbuild_sub44_mests	= build_sub44_mests_mmx;
-		pmblocks_sub44_mests = mblocks_sub44_mests_mmxe;
-	}
-	else if(cpucap & ACCEL_X86_MMX) /* Ordinary MMX CPU */
-	{
-		mjpeg_info( "SETTING MMX for MOTION!");
-		psad_sub22 = sad_sub22_mmx;
-		psad_sub44 = sad_sub44_mmx;
-		psad_00 = sad_00_mmx;
-		psad_01 = sad_01_mmx;
-		psad_10 = sad_10_mmx;
-		psad_11 = sad_11_mmx;
-		pbsad = bsad_mmx;
-		pvariance = variance_mmx;
-		psumsq = sumsq_mmx;
-		pbsumsq = bsumsq_mmx;
-		psumsq_sub22 = sumsq_sub22_mmx;
-		pbsumsq_sub22 = bsumsq_sub22_mmx;
-		pfind_best_one_pel = find_best_one_pel;
-		pbuild_sub22_mests	= build_sub22_mests;
-		pbuild_sub44_mests	= build_sub44_mests_mmx;
-		pmblocks_sub44_mests = mblocks_sub44_mests_mmx;
+	enable_mmxsse_motion(cpucap);
+#endif
+#ifdef HAVE_ALTIVEC
+	if (cpucap > 0) {
+		enable_altivec_motion();
 	}
 #endif
-
 }
