@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,22 +28,23 @@ struct DNSR_VECTOR varray22[8];
  * Lowpass-filtering is important, as this subsampled                        *
  * image is used for motion estimation and the result                        *
  * is more reliable when filtered.                                           *
+ * only subsample actual data, but keeping full buffer size for simplicity   *
  *****************************************************************************/
 
 void
-subsample_frame (uint8_t * dst[3], uint8_t * src[3])
+subsample_frame (uint8_t * dst[3], uint8_t * src[3], int w, int h)
 {
   int x, y;
-  int w=denoiser.frame.w;
-  int h=denoiser.frame.h;
+  int Cw=w/SS_H;
+  int Ch=h/SS_V;
  
   uint8_t *s  = src[0];
-  uint8_t *s2 = src[0]+w;
+  uint8_t *s2 = src[0]+W;
   uint8_t *d  = dst[0];
 
   /* subsample Y component */
 
-  for (y = 0; y < ((h+64)>>1); y += 1)
+  for (y = 0; y < (h>>1); y += 1)
   {
       for (x = 0; x < w; x += 2)
       {
@@ -49,53 +53,53 @@ subsample_frame (uint8_t * dst[3], uint8_t * src[3])
 		  *(s  + x    ) +
 		  *(s  + x + 1) + 
 		  *(s2 + x    ) +
-		  *(s2 + x + 1) )>>2;
+		  *(s2 + x + 1) + 2 )>>2;
       }
-      s+=w<<1;
-      s2+=w<<1;
-      d+=w;
+      s+=2*W;
+      s2+=2*W;
+      d+=W;
   }
 #if 1
   /* subsample U and V components */
 
   s  = src[1];
-  s2 = src[1]+(w>>1);
+  s2 = src[1]+W2;
   d  = dst[1];
 
-  for (y = 0; y < ((h+64)>>2); y += 1)
+  for (y = 0; y < (Ch>>1); y += 1)
   {
-      for (x = 0; x < (w>>1); x += 2)
+      for (x = 0; x < Cw; x += 2)
       {
 	  *(d + (x>>1)) = 
 	      (
 		  *(s  + x    ) +
 		  *(s  + x + 1) + 
 		  *(s2 + x    ) +
-		  *(s2 + x + 1) )>>2;
+		  *(s2 + x + 1) +2 )>>2;
       }
-      s+=w;
-      s2+=w;
-      d+=w>>1;
+      s+=2*W2;
+      s2+=2*W2;
+      d+=W2;
   }
 
   s  = src[2];
-  s2 = src[2]+(w>>1);
+  s2 = src[2]+W2;
   d  = dst[2];
 
-  for (y = 0; y < ((h+64)>>2); y += 1)
+  for (y = 0; y < (Ch>>1); y += 1)
   {
-      for (x = 0; x < (w>>1); x += 2)
+      for (x = 0; x < Cw; x += 2)
       {
 	  *(d + (x>>1)) = 
 	      (
 		  *(s  + x    ) +
 		  *(s  + x + 1) + 
 		  *(s2 + x    ) +
-		  *(s2 + x + 1) )>>2;
+		  *(s2 + x + 1) + 2 )>>2;
       }
-      s+=w;
-      s2+=w;
-      d+=w>>1;
+      s+=2*W2;
+      s2+=2*W2;
+      d+=W2;
   }
 #endif
 }
@@ -222,8 +226,8 @@ calc_SAD_uv_noaccel (uint8_t * frm, uint8_t * ref)
   int32_t Y = 0;
   uint32_t d = 0;
   
-  for(dy=0;dy<4;dy++)
-    for(dx=0;dx<4;dx++)
+  for(dy=0;dy<8/SS_V;dy++)
+    for(dx=0;dx<8/SS_H;dx++)
     {
       Y=*(frm+dx+dy*W2)-*(ref+dx+dy*W2);
       d+=(Y<0)? -Y:Y;
@@ -233,12 +237,12 @@ calc_SAD_uv_noaccel (uint8_t * frm, uint8_t * ref)
 
 /*********************************************************************
  *                                                                   *
- * SAD-function for UV with MMX                                      *
+ * SAD-function for 4:2:0 UV with MMX                                *
  *                                                                   *
  *********************************************************************/
 
 uint32_t
-calc_SAD_uv_mmx (uint8_t * frm, uint8_t * ref)
+calc_SAD_uv420_mmx (uint8_t * frm, uint8_t * ref)
 {
   static uint16_t a[4];
   
@@ -279,12 +283,61 @@ calc_SAD_uv_mmx (uint8_t * frm, uint8_t * ref)
 
 /*********************************************************************
  *                                                                   *
- * SAD-function for UV with MMXE                                     *
+ * SAD-function for 4:1:1 UV with MMX                                *
  *                                                                   *
  *********************************************************************/
 
 uint32_t
-calc_SAD_uv_mmxe (uint8_t * frm, uint8_t * ref)
+calc_SAD_uv411_mmx (uint8_t * frm, uint8_t * ref)
+{
+  static uint16_t a[8];
+  
+#ifdef HAVE_ASM_MMX
+  __asm__ __volatile__
+    (
+    " pxor        %%mm0 , %%mm0;           /* clear mm0                                          */\n"
+    " pxor        %%mm7 , %%mm7;           /* clear mm7                                          */\n"
+    "                                      /*                                                    */\n"
+    " movl         %1    , %%eax;          /* load frameadress into eax                          */\n"
+    " movl         %2    , %%ebx;          /* load frameadress into ebx                          */\n"
+    " movl         %3    , %%ecx;          /* load width       into ecx                          */\n"
+    "                                      /*                                                    */\n"
+    ".rept 8                    ;          /* Loop for 8 lines                                   */\n"
+    " movw        (%%eax),  %%dx;          /*                                                    */\n"
+    " movd         %%edx , %%mm1;          /* 2 Pixels from filtered frame to mm1                */\n"
+    " movw        (%%ebx),  %%dx;          /*                                                    */\n"
+    " movd         %%edx , %%mm2;          /* 2 Pixels from reference frame to mm2               */\n"
+    " movq         %%mm2 , %%mm3;          /* hold a copy of mm2 in mm3                          */\n"
+    " psubusb      %%mm1 , %%mm3;          /* positive differences between mm2 and mm1           */\n"
+    " psubusb      %%mm2 , %%mm1;          /* positive differences between mm1 and mm3           */\n"
+    " paddusb      %%mm3 , %%mm1;          /* mm2 now contains abs(mm1-mm2)                      */\n"
+    " movq         %%mm1 , %%mm2;          /* copy mm1 to mm2                                    */\n"
+    " punpcklbw    %%mm7 , %%mm1;          /* unpack mm1 into mm1 and mm2                        */\n"
+    " punpckhbw    %%mm7 , %%mm2;          /*                                                    */\n"
+    " paddusw      %%mm1 , %%mm2;          /* add mm1 (stored in mm1 and mm2...)                 */\n"
+    " paddusw      %%mm2 , %%mm0;          /* to mm0                                             */\n"
+    " addl         %%ecx , %%eax;          /* add framewidth to frameaddress                     */\n"
+    " addl         %%ecx , %%ebx;          /* add framewidth to frameaddress                     */\n"
+    " .endr                                /* end loop                                           */\n"
+    "                                      /*                                                    */\n"
+    " movq         %%mm0 , %0   ;          /* make mm0 available to gcc ...                      */\n"
+    :"=m" (a)     
+    :"m" (frm), "m" (ref), "m" (W2)
+    :"%eax", "%ebx", "%ecx"
+    );
+#endif
+  return (uint32_t)(a[0]+a[1]+a[2]+a[3]+a[4]+a[5]+a[6]+a[7]);
+}
+
+
+/*********************************************************************
+ *                                                                   *
+ * SAD-function for 4:2:0 UV with MMXE                               *
+ *                                                                   *
+ *********************************************************************/
+
+uint32_t
+calc_SAD_uv420_mmxe (uint8_t * frm, uint8_t * ref)
 {
   static uint32_t a;
 
@@ -314,6 +367,45 @@ calc_SAD_uv_mmxe (uint8_t * frm, uint8_t * ref)
   return a;
 }
 
+/*********************************************************************
+ *                                                                   *
+ * SAD-function for 4:1:1 UV with MMXE                               *
+ *                                                                   *
+ *********************************************************************/
+
+uint32_t
+calc_SAD_uv411_mmxe (uint8_t * frm, uint8_t * ref)
+{
+  static uint32_t a;
+
+#ifdef HAVE_ASM_MMX
+  __asm__ __volatile__
+    (
+    " pxor         %%mm0 , %%mm0;          /* clear mm0                                          */\n"
+    " movl         %1    , %%eax;          /* load frameadress into eax                          */\n"
+    " movl         %2    , %%ebx;          /* load frameadress into ebx                          */\n"
+    " movl         %3    , %%ecx;          /* load width       into ecx                          */\n"
+    "                           ;          /*                                                    */\n"
+    " .rept 8                   ;          /*                                                    */\n"
+    " movw        (%%eax),  %%dx;          /*                                                    */\n"
+    " movd         %%edx , %%mm1;          /* 2 Pixels from filtered frame to mm1                */\n"
+    " movw        (%%ebx),  %%dx;          /*                                                    */\n"
+    " movd         %%edx , %%mm2;          /* 2 Pixels from filtered frame to mm2                */\n"
+    " psadbw       %%mm2 , %%mm1;          /* 2 Pixels difference to mm1                         */\n"
+    " paddusw      %%mm1 , %%mm0;          /* add result to mm0                                  */\n"
+    " addl         %%ecx , %%eax;          /* add framewidth to frameaddress                     */\n"
+    " addl         %%ecx , %%ebx;          /* add framewidth to frameaddress                     */\n"
+    " .endr                     ;          /*                                                    */\n"
+    "                                      /*                                                    */\n"
+    " movq         %%mm0 , %0   ;          /* make mm0 available to gcc ...                      */\n"
+    :"=m" (a)     
+    :"m" (frm), "m" (ref), "m" (W2)
+    :"%eax", "%ebx", "%ecx"
+    );
+#endif
+  return a;
+}
+
 /*********************************************************************
  *                                                                   *
  * halfpel SAD-function for Y without MMX/MMXE                       *
@@ -443,7 +535,7 @@ mb_search_44 (uint16_t x, uint16_t y)
   uint8_t  radius = denoiser.radius>>2;       /* search radius /4 in pixels */
   int32_t  MB_ref_offset = denoiser.frame.w * (y>>2) + (x>>2);
   int32_t  MB_avg_offset;
-  int32_t  MB_ref_offset_uv = (denoiser.frame.w>>1) * (y>>3) + (x>>3);
+  int32_t  MB_ref_offset_uv = (denoiser.frame.Cw) * (y>>2)/SS_V + (x>>2)/SS_H;
   int32_t  MB_avg_offset_uv;
   int32_t  last_uv_offset=0;
   int16_t  xx;
@@ -457,11 +549,11 @@ mb_search_44 (uint16_t x, uint16_t y)
   SAD += calc_SAD_uv ( denoiser.frame.sub4ref[2]+MB_ref_offset_uv, 
                        denoiser.frame.sub4avg[2]+MB_ref_offset_uv );
   
-  for(yy=-radius;yy<radius;yy++)
-    for(xx=-radius;xx<radius;xx++)
+  for(yy=-radius;yy<=radius;yy++)
+    for(xx=-radius;xx<=radius;xx++)
     {
       MB_avg_offset    = MB_ref_offset+(xx)+(yy*denoiser.frame.w);
-      MB_avg_offset_uv = MB_ref_offset_uv+(xx>>1)+((yy>>1)*(denoiser.frame.w>>1));
+      MB_avg_offset_uv = MB_ref_offset_uv+(xx/SS_H)+((yy/SS_V)*denoiser.frame.Cw);
       
       SAD = calc_SAD ( denoiser.frame.sub4ref[0]+MB_ref_offset, 
                        denoiser.frame.sub4avg[0]+MB_avg_offset );
@@ -502,7 +594,7 @@ mb_search_22 (uint16_t x, uint16_t y)
   uint32_t SAD_uv=0x00ffffff; 
   int32_t  MB_ref_offset = denoiser.frame.w * (y>>1) + (x>>1);
   int32_t  MB_avg_offset;
-  int32_t  MB_ref_offset_uv = (denoiser.frame.w>>1) * (y>>2) + (x>>2);
+  int32_t  MB_ref_offset_uv = (denoiser.frame.Cw) * (y>>1)/SS_V + (x>>1)/SS_H;
   int32_t  MB_avg_offset_uv;
   int32_t  last_uv_offset=0;
   int16_t  xx;
@@ -512,11 +604,12 @@ mb_search_22 (uint16_t x, uint16_t y)
     
   /* motion-vectors from 44 can/will be wrong by +/- 3 pixels */
     
-  for(yy=-2;yy<2;yy++)
-    for(xx=-2;xx<2;xx++)
+  for(yy=-2;yy<=2;yy++)
+    for(xx=-2;xx<=2;xx++)
     {
       MB_avg_offset=MB_ref_offset+(xx+vx)+((yy+vy)*denoiser.frame.w);
-      MB_avg_offset_uv = MB_ref_offset_uv+((xx+vx)>>2)+((yy+vy)>>2)*(denoiser.frame.w>>1);
+      /* MB_avg_offset_uv = MB_ref_offset_uv+((xx+vx)>>1)/SS_H+(((yy+vy)>>1)/SS_V)*(denoiser.frame.Cw); */
+      MB_avg_offset_uv = MB_ref_offset_uv+((xx+vx)/SS_H)+((yy+vy)/SS_V)*(denoiser.frame.Cw);
       
       SAD = calc_SAD ( denoiser.frame.sub2ref[0]+MB_ref_offset, 
                        denoiser.frame.sub2avg[0]+MB_avg_offset );
@@ -559,8 +652,12 @@ mb_search_11 (uint16_t x, uint16_t y)
 {
   uint32_t best_SAD = 0x00ffffff;
   uint32_t SAD=0x00ffffff; 
+  uint32_t SAD_uv=0x00ffffff; 
   int32_t  MB_ref_offset = denoiser.frame.w * (y) + (x);
   int32_t  MB_avg_offset;
+  int32_t  MB_ref_offset_uv = (denoiser.frame.Cw) * y/SS_V + x/SS_H;
+  int32_t  MB_avg_offset_uv;
+  int32_t  last_uv_offset=0;
   int16_t  xx;
   int16_t  yy;
   int16_t  vx=vector.x<<1;
@@ -568,14 +665,25 @@ mb_search_11 (uint16_t x, uint16_t y)
   
   /* motion-vectors from 22 can/will be wrong by +/- 2 pixels */
 
-  for(yy=-2;yy<2;yy++)
-    for(xx=-2;xx<2;xx++)
+  for(yy=-2;yy<=2;yy++)
+    for(xx=-2;xx<=2;xx++)
     {
       MB_avg_offset=MB_ref_offset+(xx+vx)+((yy+vy)*denoiser.frame.w);
+      MB_avg_offset_uv = MB_ref_offset_uv+((xx+vx)/SS_H)+((yy+vy)/SS_V)*(denoiser.frame.Cw);
       
         SAD = calc_SAD ( denoiser.frame.ref[0]+MB_ref_offset, 
                          denoiser.frame.avg[0]+MB_avg_offset );
-        
+ 
+      if(MB_ref_offset_uv != last_uv_offset)
+      {
+        last_uv_offset=MB_ref_offset_uv;
+        SAD_uv = calc_SAD_uv ( denoiser.frame.sub2ref[1]+MB_ref_offset_uv, 
+                               denoiser.frame.sub2avg[1]+MB_avg_offset_uv );
+        SAD_uv += calc_SAD_uv ( denoiser.frame.sub2ref[2]+MB_ref_offset_uv, 
+                                denoiser.frame.sub2avg[2]+MB_avg_offset_uv );
+      }
+      SAD += SAD_uv;
+       
       if(SAD<best_SAD)
       {
         best_SAD = SAD;
@@ -620,8 +728,8 @@ mb_search_00 (uint16_t x, uint16_t y)
   
   MB_avg_offset1=MB_ref_offset+(vx)+((vy)*denoiser.frame.w);
 
-  for(yy=-1;yy<1;yy++)
-    for(xx=-1;xx<1;xx++)
+  for(yy=-1;yy<=1;yy++)
+    for(xx=-1;xx<=1;xx++)
     {
       MB_avg_offset2=MB_ref_offset+(vx+xx)+((vy+yy)*denoiser.frame.w);
       
